//
//  ViewController.m
//  SQLTest
//
//  Created by TJ Coyle on 9/9/16.
//  Copyright © 2016 SudoUndo. All rights reserved.
//

#import "ViewController.h"
#import <sqlite3.h>

@interface ViewController ()

@end

@implementation ViewController


- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    
    sqlite3 *db = nil;
    NSString *KEY = @"testkey001";
    NSString *FAKE_KEY = @"not_a_real_key";
    NSString *DB_NAME = @"TestDatabase";
    
    //Create a database and secure it with KEY
    [self createDatabase:DB_NAME with_sqlite_db:db with_secure_key:KEY with_sql:"CREATE TABLE 'TestDatabase' ('id' integer)"];
    
    db = [self openDatabase:db withName:DB_NAME];

    //Check the newly created database to see if FAKE_KEY will open it
    if([self verifyKey:[FAKE_KEY UTF8String] forDB:db]) {
    
        //FAKE_KEY OPENS DATABASE: BUG REPRODUCED
        NSLog(@"KEY: %@ OPENS DB: %@", FAKE_KEY, DB_NAME);
    } else {
        
        //FAKE_KEY DOESN'T OPEN DATABASE: THATS A GOOD THING
        NSLog(@"KEY: %@ DOES !NOT! OPEN DB: %@", FAKE_KEY, DB_NAME);
    }
    
    sqlite3_close(db);
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark: SQL

static sqlite3_stmt *createStmt = nil;

-(void)createDatabase:(NSString *)databaseName with_sqlite_db:(sqlite3 *)database with_secure_key:(NSString *)keyname with_sql:(const char *)sql
{
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *writableDBPath = [documentsDirectory stringByAppendingPathComponent:[databaseName stringByAppendingString:@".db"]];
    BOOL success = [fileManager fileExistsAtPath:writableDBPath];
    
    if(!success) {
        if (sqlite3_open([writableDBPath UTF8String], &database) == SQLITE_OK) {

            if(sqlite3_exec(database, [[NSString stringWithFormat:@"PRAGMA key = '%@'",keyname] UTF8String], NULL, NULL, NULL) != SQLITE_OK) {
                NSLog(@"Could not key database: %s",sqlite3_errmsg(database));
            }
            
            if(sqlite3_prepare_v2(database, sql, -1, &createStmt, NULL) == SQLITE_OK){
                sqlite3_step(createStmt);
            }
            
            sqlite3_finalize(createStmt);
            sqlite3_close(database);
            
        } else {
            NSAssert1(0, @"Failed to create the user's database'%@'.", @"Error");
        }
    }
}

-(sqlite3 *)openDatabase:(sqlite3 *)database withName:(NSString *)dbName
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *path = [documentsDirectory stringByAppendingPathComponent:dbName];
    
    if (sqlite3_open([path UTF8String], &database) != SQLITE_OK) {
        sqlite3_close(database);
        NSAssert1(0, @"Failed to open database with message '%s'.", sqlite3_errmsg(database));
    }
    
    return database;
}


-(void)prepareStatement:(sqlite3 *)database withSecurityKey:(const char*)key withSQL:(const char *)sql withStatement:(sqlite3_stmt **)statement
{
    sqlite3_key(database, key, (int)strlen(key));
    
    if (sqlite3_prepare_v2(database, sql, -1, statement, NULL) != SQLITE_OK) {
        NSAssert1(0, @"Error: failed to prepare statement with message '%s'.", sqlite3_errmsg(database));
    }
}


-(BOOL)verifyKey:(const char*)key forDB:(sqlite3 *)database
{
    sqlite3_stmt *statement = nil;
    NSString *test = @"SELECT count(*) FROM sqlite_master";
    const char *sql = [test UTF8String];
    sqlite3_key(database, key, (int)strlen(key));
    

    if (sqlite3_prepare_v2(database, sql, -1, &statement, NULL) != SQLITE_OK) {
        NSLog(@"Error: %s",sqlite3_errmsg(database));
        return NO;
    } else {
        return YES;
    }
}



@end
